import pandas as pd
import numpy as np
from xgboost import XGBRegressor
from lightgbm import LGBMRegressor
from catboost import CatBoostRegressor
from sklearn.model_selection import train_test_split
from sklearn.model_selection import RandomizedSearchCV
from sklearn.metrics import roc_auc_score
from sklearn.metrics import mean_squared_error as MSE
import shap

# extract data
features = pd.read_csv("C:\literature\XGB_DATA_BLANK0.csv")
test_feature_LSR = pd.read_csv("C:\literature\LSR.csv")
names = [column for column in features]

X = features[names[0:-2]]
YOS = features['YOS']
YOA = features['YOA']

LSR = test_feature_LSR[names[0:-2]]


# hyperparameter adjusting

param_grid = {'max_depth': range(2, 15, 1),
              'n_estimators': range(40, 640, 5),
              'learning_rate': [0.01, 0.02, 0.03, 0.04, 0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5],
              "gamma": [0.0, 0.1, 0.2, 0.3, 0.4],
              "reg_alpha": [0.0001, 0.001, 0.01, 0.1, 1],
              "reg_lambda": [0.0001, 0.001, 0.01, 0.1, 1],
              "min_child_weight": [1, 2, 3, 4, 5, 6, 7, 8],
              "colsample_bytree": [0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1],
              "subsample": [0.6, 0.7, 0.8, 0.9, 1]
              }
x_train, x_test, y_train, y_test = train_test_split(X, YOS, test_size=0.3, random_state=2)
xgbr = XGBRegressor().fit(x_train, y_train)

glf = RandomizedSearchCV(estimator=xgbr,
                             param_distributions=param_grid,
                             cv=5, n_iter=50,
                             scoring='neg_mean_absolute_error',
                             n_jobs=4,
                             verbose=5,
                             return_train_score=True,
                             random_state=2)

glf.fit(x_train, y_train)

# print(glf.best_params_, glf.best_score_)


XGBR = XGBRegressor(subsample=glf.best_params_['subsample'],
                    reg_lambda=glf.best_params_['reg_lambda'],
                    reg_alpha=glf.best_params_['reg_alpha'],
                    n_estimators=glf.best_params_['n_estimators'],
                    min_child_weight=glf.best_params_['min_child_weight'],
                    max_depth=glf.best_params_['max_depth'],
                    learning_rate=glf.best_params_['learning_rate'],
                    gamma=glf.best_params_['gamma'],
                    colsample_bytree=glf.best_params_['colsample_bytree'],
                    ).fit(x_train, y_train)

y_pred = XGBR.predict(x_test)
print('Xgboost prediction on yield of solid : ','\n'
      'R2_train', XGBR.score(x_train, y_train), '\n'
      'R2_test', XGBR.score(x_test, y_test), '\n'
      'actual', list(y_test), '\n'
      'predictions', y_pred)


# split datasets into train sets and test sets
x_train, x_test, y_train, y_test = train_test_split(X, YOS, test_size=0.3, random_state=2)

XGBR = XGBRegressor(subsample=1,
                    reg_lambda=0.1,
                    reg_alpha=0.01,
                    n_estimators=495,
                    min_child_weight=5,
                    max_depth=9,
                    learning_rate=0.15,
                    gamma=0.2,
                    colsample_bytree=0.9,
                    ).fit(x_train, y_train)

y_pred = XGBR.predict(x_test)
print('Xgboost prediction on yield of solid : ', '\n'
                                                 'R2_train', XGBR.score(x_train, y_train), '\n'
                                                                                           'R2_test',
      XGBR.score(x_test, y_test), '\n'
                                  'actual', list(y_test), '\n'
                                                          'predictions', y_pred)
###
###  repeat same process on yield of aromatics
###
x_train, x_test, y_train, y_test = train_test_split(X, YOA, test_size=0.3, random_state=66)

XGBR = XGBRegressor(subsample=0.6,
                    reg_lambda=0.001,
                    reg_alpha=0.0001,
                    n_estimators=355,
                    min_child_weight=2,
                    max_depth=6,
                    learning_rate=0.02,
                    gamma=0.4,
                    colsample_bytree=0.5,
                    ).fit(x_train, y_train)

y_pred = XGBR.predict(x_test)
print('Xgboost prediction on yield of aromatics : ', '\n'
                                                     'R2_train', XGBR.score(x_train, y_train), '\n'
                                                                                               'R2_test',
      XGBR.score(x_test, y_test), '\n'
                                  'actual', list(y_test), '\n'
                                                          'predictions', y_pred)

# split datasets into train sets and test sets
x_train, x_test, y_train, y_test = train_test_split(X, YOS, test_size=0.3, random_state=2)

LGB = LGBMRegressor(
        learning_rate = 0.25,
        n_estimators=455,
        max_depth=6,
        num_leaves= 360,
        min_child_weight=0.002,
        min_child_samples=15,
        bagging_fraction=1.0,
        reg_alpha=0.1
).fit(x_train, y_train)

y_pred = LGB.predict(x_test)
print('LGBM prediction on yield of solid : ', '\n'
                                                 'R2_train', LGB.score(x_train, y_train), '\n'
                                                                                           'R2_test',
      LGB.score(x_test, y_test), '\n'
                                  'actual', list(y_test), '\n'
                                                          'predictions', y_pred)
###
###  repeat same process on yield of aromatics
###
x_train, x_test, y_train, y_test = train_test_split(X, YOA, test_size=0.3, random_state=2)

LGB = LGBMRegressor(
        learning_rate = 0.2,
        n_estimators=505,
        max_depth=12,
        num_leaves= 40,
        min_child_weight=0.0001,
        min_child_samples=15,
        bagging_fraction=0.65,
        reg_alpha=0.1
).fit(x_train, y_train)

y_pred = LGB.predict(x_test)
print('LGBM prediction on yield of aromatics : ', '\n'
                                                     'R2_train', LGB.score(x_train, y_train), '\n'
                                                                                               'R2_test',
      LGB.score(x_test, y_test), '\n'
                                  'actual', list(y_test), '\n'
                                                          'predictions', y_pred)

x_train, x_test, y_train, y_test = train_test_split(X, YOS, test_size=0.3, random_state=5)

CATB = CatBoostRegressor(
    iterations=305,
    l2_leaf_reg=50,
    max_depth=6,
    learning_rate=1
).fit(x_train, y_train)

y_pred = CATB.predict(x_test)
print('CatBoost prediction on yield of solid : ', '\n'
                                                 'R2_train', CATB.score(x_train, y_train), '\n'
                                                                                           'R2_test',
      CATB.score(x_test, y_test), '\n'
                                  'actual', list(y_test), '\n'
                                                          'predictions', y_pred)
###
###  repeat same process on yield of aromatics
###
x_train, x_test, y_train, y_test = train_test_split(X, YOA, test_size=0.3, random_state=16)

CATB = CatBoostRegressor(
    iterations=540,
    l2_leaf_reg=10,
    max_depth=6,
    learning_rate=0.1
).fit(x_train, y_train)

y_pred = CATB.predict(x_test)
print('CatBoost prediction on yield of aromatics : ', '\n'
                                                     'R2_train', CATB.score(x_train, y_train), '\n'
                                                                                               'R2_test',
      CATB.score(x_test, y_test), '\n'
                                  'actual', list(y_test), '\n'
                                                          'predictions', y_pred)

###
###  SHAP output
###

explainer = shap.TreeExplainer(CATB)
shap_values = explainer.shap_values(X)
shap.summary_plot(shap_values, X)
shap.summary_plot(shap_values, X, plot_type='bar')

###
###  simulation
###

y_Simulation_pred = XGBR.predict(LSR)

print(y_Simulation_pred)